package com.backendsys.modules.system.service;

import com.backendsys.modules.system.entity.*;
import com.backendsys.utils.response.PageEntity;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.Map;

public interface SysUserService extends IService<SysUser> {

    // 获取系统用户列表
    PageEntity selectUserList(SysUserDTO sysUserDTO);
    // 获取系统用户列表 (在线的)
    PageEntity selectUserOnlineList(SysUserDTO sysUserDTO);

    // 获取系统用户详情
    SysUserInfo selectUserInfo(Long user_id);
    // 获取系统用户详情(简约)
    SysUserInfoSimple selectUserInfoSimple(Long user_id);

    // 获取系统用户权限
    Map<String, Object> selectUserPermission(Long user_id);
    // 获取系统用户菜单
    Map<String, Object> selectUserMenu(Long user_id);
    // 创建系统用户
    Map<String, Object> insertUser(SysUserDTO sysUserDTO);
    // 编辑系统用户信息
    Map<String, Object> updateUserInfo(SysUserDTO sysUserDTO);
    // 编辑系统用户角色绑定
    Map<String, Object> updateUserRoleRelation(SysUserRoleRelation sysUserRoleRelation);
    // 编辑系统用户密码
    Map<String, Object> updateUserPassword(SysUserDTO sysUserDTO);
    // 重置系统用户密码
    Map<String, Object> resetUserPassword(SysUserDTO sysUserDTO);
    // 删除系统用户
    Map<String, Object> deleteUser(SysUserDTO sysUserDTO);
    // 踢出系统用户
    Map<String, Object> kickUser(Long user_id);
    // 审核用户
    Map<String, Object> auditUser(SysUserDTO sysUserDTO);

}
